@@ -37,6 +37,7 @@ from fabric.api import (
 from fabric.colors import red
 from fabric.contrib.files import append, exists, sed, uncomment, upload_template
 from fabric.exceptions import NetworkError
+from fabric.main import list_commands
 from fabric.network import disconnect_all
 
 from fabulaws.api import answer_sudo, ec2_instances, sshagent_run
@@ -72,7 +73,7 @@ env.role_class_map = {
 }
 
 config_file = "fabulaws-config.yml"
-config = yaml.load(open(config_file, "r"))
+config = yaml.full_load(open(config_file, "r"))
 for key, value in config.items():
     setattr(env, key, value)
 _reset_hosts()
@@ -504,7 +505,7 @@ def _new(
     terminate_on_failure=False,
     **kwargs
 ):
-    """ create new server on AWS using the given deployment, environment, and role """
+    """create new server on AWS using the given deployment, environment, and role"""
     if deployment not in env.deployments:
         abort("Choose a valid deployment: %s" % ", ".join(env.deployments))
     if environment not in env.environments:
@@ -728,7 +729,7 @@ def vcs(cmd, args=None):
 @runs_once
 @roles("worker")
 def update_local_fabsecrets():
-    """ create or update the local fabsecrets_<environment>.py file based on the passwords on the server """
+    """create or update the local fabsecrets_<environment>.py file based on the passwords on the server"""
 
     require("environment", provided_by=env.environments)
 
@@ -762,7 +763,7 @@ def update_local_fabsecrets():
 @parallel
 @roles("web", "worker")
 def clone_repo():
-    """ clone a new copy of the code repository """
+    """clone a new copy of the code repository"""
 
     with cd(env.root):
         vcs("clone", [env.repo, env.code_root])
@@ -774,7 +775,7 @@ def clone_repo():
 @parallel
 @roles("web", "worker")
 def setup_dirs():
-    """ create (if necessary) and make writable uploaded media, log, etc. directories """
+    """create (if necessary) and make writable uploaded media, log, etc. directories"""
 
     require("environment", provided_by=env.environments)
     sudo("mkdir -p %(log_dir)s" % env, user=env.deploy_user)
@@ -795,9 +796,12 @@ def setup_dirs():
 def _upload_template(filename, destination, **kwargs):
     """Upload template and chown to given user"""
     user = kwargs.pop("user")
+    mode = kwargs.pop("mode", None)
     kwargs["use_sudo"] = True
     upload_template(filename, destination, **kwargs)
     sudo("chown %(user)s:%(user)s %(dest)s" % {"user": user, "dest": destination})
+    if mode:
+        sudo("chmod %(mode)s %(dest)s" % {"mode": mode, "dest": destination})
 
 
 @task
@@ -807,6 +811,7 @@ def upload_supervisor_conf(run_update=True):
     """Upload Supervisor configuration from the template."""
 
     require("environment", provided_by=env.environments)
+    _load_passwords(["database_password"])
     destination = os.path.join(env.services, "supervisor", "%(environment)s.conf" % env)
     context = env.copy()
     cpu_count = int(run("cat /proc/cpuinfo|grep processor|wc -l"))
@@ -816,6 +821,10 @@ def upload_supervisor_conf(run_update=True):
         getattr(env, "gunicorn_worker_multiplier", 4)
     )
     context["current_role"] = _current_roles()[0]
+    entrypoint_dest = os.path.join(
+        env.services, "supervisor", "gunicorn-%(environment)s-entrypoint.sh" % env
+    )
+    context["gunicorn_entrypoint"] = entrypoint_dest
     _upload_template(
         "supervisor.conf",
         destination,
@@ -824,6 +833,13 @@ def upload_supervisor_conf(run_update=True):
         use_jinja=True,
         template_dir=env.templates_dir,
     )
+    _upload_template(
+        "gunicorn-entrypoint.sh",
+        entrypoint_dest,
+        user=env.deploy_user,
+        mode="0755",
+        template_dir=env.templates_dir,
+    )
     with settings(warn_only=True):
         sudo("rm /etc/supervisor/conf.d/%(project)s-*.conf" % env)
     sudo(
@@ -983,7 +999,7 @@ def upload_newrelic_conf():
 @parallel
 @roles("web", "worker")
 def update_services():
-    """ upload changes to services configurations as nginx """
+    """upload changes to services configurations as nginx"""
 
     setup_dirs()
     upload_newrelic_conf()
@@ -997,7 +1013,7 @@ def update_services():
 @parallel
 @roles("web", "worker")
 def create_virtualenv():
-    """ setup virtualenv on remote host """
+    """setup virtualenv on remote host"""
 
     require("virtualenv_root", provided_by=env.environments)
     cmd = ["virtualenv", "--clear", "--python=%(python)s" % env, env.virtualenv_root]
@@ -1008,7 +1024,7 @@ def create_virtualenv():
 @parallel
 @roles("web", "worker")
 def update_requirements():
-    """ update external dependencies on remote host """
+    """update external dependencies on remote host"""
 
     require("code_root", provided_by=env.environments)
     # add HOME= so if there's an error, pip can save the log (Fabric doesn't
@@ -1026,7 +1042,7 @@ def update_requirements():
 @parallel
 @roles("web", "worker")
 def update_local_settings():
-    """ create local_settings.py on the remote host """
+    """create local_settings.py on the remote host"""
 
     require("environment", provided_by=env.environments)
     _load_passwords(env.password_names)
@@ -1056,7 +1072,7 @@ def update_local_settings():
 @parallel
 @roles("web", "worker")
 def bootstrap(purge=False):
-    """ initialize remote host environment (virtualenv, deploy, update) """
+    """initialize remote host environment (virtualenv, deploy, update)"""
 
     require("environment", provided_by=env.environments)
 
@@ -1068,6 +1084,11 @@ def bootstrap(purge=False):
     create_virtualenv()
     update_requirements()
 
+    # Run a "post_bootstrap" task, if it's defined.
+    available_commands = list_commands("", "short")
+    if "post_bootstrap" in available_commands:
+        executel("post_bootstrap", hosts=[env.host])
+
 
 # CODE DEPLOYMENT
 
@@ -1194,7 +1215,7 @@ def dbrestore(filepath):
         put(private_key, dest)
         with settings(warn_only=True):
             sudo(
-                'gpg --homedir {0} --batch --delete-secret-keys "{1}"'.format(
+                'gpg --homedir {0} --batch --yes --delete-secret-keys "{1}"'.format(
                     env.gpg_dir, env.backup_key_fingerprint
                 ),
                 user=env.webserver_user,
@@ -1227,7 +1248,7 @@ def dbrestore(filepath):
     finally:
         with settings(warn_only=True):
             sudo(
-                'gpg --homedir {0} --batch --delete-secret-keys "{1}"'.format(
+                'gpg --homedir {0} --batch --yes --delete-secret-keys "{1}"'.format(
                     env.gpg_dir, env.backup_key_fingerprint
                 ),
                 user=env.webserver_user,
@@ -1416,7 +1437,7 @@ def add_all_to_elb():
 @task
 @roles("db-primary")  # only supported on combo web & database servers
 def reload_production_db(prod_env=env.default_deployment, src_env="production"):
-    """ Replace the testing or staging database with the production database """
+    """Replace the testing or staging database with the production database"""
     if env.environment not in ("staging", "testing"):
         abort("prod_to_staging requires the staging or testing environment.")
     executel("suspend_autoscaling_processes", env.deployment_tag, env.environment)
@@ -1438,7 +1459,8 @@ def reload_production_db(prod_env=env.default_deployment, src_env="production"):
         )
     )
     load_cmd = 'bash -o pipefail -c "{dump_cmd} | psql {db_name}"'.format(
-        dump_cmd=dump_cmd, db_name=env.database_name,
+        dump_cmd=dump_cmd,
+        db_name=env.database_name,
     )
     sshagent_run(load_cmd, user=env.deploy_user)
     executel("supervisor", "start", "pgbouncer")
@@ -1448,11 +1470,16 @@ def reload_production_db(prod_env=env.default_deployment, src_env="production"):
     executel("end_upgrade")
     executel("resume_autoscaling_processes", env.deployment_tag, env.environment)
 
+    # Run a "post_reload_production_db" task, if it's defined.
+    available_commands = list_commands("", "short")
+    if "post_reload_production_db" in available_commands:
+        executel("post_reload_production_db", hosts=[env.host])
+
 
 @task
 @roles("db-primary")
 def reset_local_db(db_name):
-    """ Replace the local database with the remote database """
+    """Replace the local database with the remote database"""
 
     require("environment", provided_by=env.environments)
     answer = prompt(
@@ -1468,7 +1495,9 @@ def reset_local_db(db_name):
         local("dropdb {0}".format(db_name))
     local("createdb {0}".format(db_name))
     cmd = "ssh -C {user}@{host} pg_dump -Ox {db_name} | ".format(
-        user=env.deploy_user, host=env.host_string, db_name=env.database_name,
+        user=env.deploy_user,
+        host=env.host_string,
+        db_name=env.database_name,
     )
     cmd += "psql {0}".format(db_name)
     local(cmd)
@@ -1535,14 +1564,20 @@ def update_sysadmin_users():
 
 @task
 def upgrade_packages():
-    """ update packages on the servers """
+    """update packages on the servers"""
 
     with settings(warn_only=True):
-        sudo("apt-get -qq update || apt-get -qq update")
+        sudo(
+            "export DEBIAN_FRONTEND=noninteractive ; apt-get -qq update || apt-get -qq update"
+        )
     if "web" in _current_roles() or "worker" in _current_roles():
         packages = env.app_server_packages
-        sudo("apt-get -qq -y install {0}".format(" ".join(packages)))
-    sudo("apt-get -qq -y upgrade")
+        sudo(
+            "export DEBIAN_FRONTEND=noninteractive ; apt-get -qq -y install {0}".format(
+                " ".join(packages)
+            )
+        )
+    sudo("export DEBIAN_FRONTEND=noninteractive ; apt-get -qq -y upgrade")
 
 
 @task
@@ -1920,7 +1955,9 @@ def recreate_servers(deployment_tag, environment, wait=30):
 @parallel
 def install_munin():
     require("environment", provided_by=env.environments)
-    sudo("apt-get -qq -y install munin-node munin-plugins-extra libdbd-pg-perl")
+    sudo(
+        "export DEBIAN_FRONTEND=noninteractive ; apt-get -qq -y install munin-node munin-plugins-extra libdbd-pg-perl"
+    )
     append("/etc/munin/munin-node.conf", "cidr_allow 10.0.0.0/8", use_sudo=True)
     append("/etc/munin/munin-node.conf", "cidr_allow 172.16.0.0/12", use_sudo=True)
     append("/etc/munin/munin-node.conf", "cidr_allow 192.168.0.0/16", use_sudo=True)
@@ -1930,6 +1967,11 @@ def install_munin():
 @task
 @parallel
 def install_rsyslog():
+    # Run a "pre_install_rsyslog" task, if it's defined.
+    available_commands = list_commands("", "short")
+    if "pre_install_rsyslog" in available_commands:
+        executel("pre_install_rsyslog", hosts=[env.host])
+
     require("environment", provided_by=env.environments)
     context = dict(env)
     context["current_role"] = _current_roles()[0]
@@ -1947,10 +1989,14 @@ def install_rsyslog():
 
     output = run("rsyslogd -v")
     if "rsyslogd 8" not in output:
-        sudo("add-apt-repository --yes ppa:adiscon/v8-stable")
+        sudo(
+            "export DEBIAN_FRONTEND=noninteractive ; add-apt-repository --yes ppa:adiscon/v8-stable"
+        )
         with settings(warn_only=True):
-            sudo("apt-get -qq update || apt-get -qq update")
-        sudo("apt-get -qq -y install rsyslog")
+            sudo(
+                "export DEBIAN_FRONTEND=noninteractive ; apt-get -qq update || apt-get -qq update"
+            )
+        sudo("export DEBIAN_FRONTEND=noninteractive ; apt-get -qq -y install rsyslog")
 
     print("Ignore any useradd or chgrp warnings below.")
     with settings(warn_only=True):
@@ -1981,7 +2027,9 @@ def install_logstash():
     _upload_template(template, destination, user="root", context=context)
     with (cd("/tmp")):
         if not exists("logstash.jar"):
-            sudo("apt-get -qq -y install default-jre")
+            sudo(
+                "export DEBIAN_FRONTEND=noninteractive ; apt-get -qq -y install default-jre"
+            )
             sudo(
                 "wget https://download.elasticsearch.org/logstash/logstash/logstash-1.1.7-monolithic.jar -O logstash.jar",
                 user=env.deploy_user,
@@ -2027,7 +2075,7 @@ def install_awslogs():
             user=env.deploy_user,
         )
         # This script only supports Python 2.6 - 3.5, so make sure Python 2.7 is installed (any Python 3 version is likely too new)
-        sudo("apt-get install -y python2.7")
+        sudo("export DEBIAN_FRONTEND=noninteractive ; apt-get install -y python2.7")
         sudo(
             "python2.7 awslogs-agent-setup.py --region us-east-1 --non-interactive --configfile awslogs.conf"
         )
